Package: OpenRange
Title: Code to Access Open Access Species Range Maps
Version: 0.0.2
Authors@R: 
    c(
      person("Brian", "Maitner", , "bmaitner@usf.edu", role = c("aut", "cre"),
      comment = c(ORCID = "https://orcid.org/0000-0002-2118-9880")),
    person("Cory", "Merow", , "cory.merow@gmail.com", role = c("aut")),
    person("Brad", "Boyle", , "bboyle@arizona.edu", role = c("aut")),
    person("Xiao", "Feng", , "xiao.feng@unc.edu", role = c("aut")),
    person("Rethvick Sriram Yugendra", "Babu", email = "rethvick@arizona.edu",
    role = c("aut")),
    person("Brian", "Enquist", , "benquist@arizona.edu", role = c("aut"))
    )
Description: Allows access to a proof-of-concept database containing Open Access species range models and relevant metadata. Access to the database is via both 'PostgreSQL' connection and API <https://github.com/EnquistLab/Biendata-Frontend>, allowing diverse use-cases.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 4.1.0), RPostgreSQL
Imports: DBI, sf, httr, jsonlite
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), maps, tidyverse, vcr,
        ggplot2
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-08-24 18:59:11 UTC; bmaitner
Author: Brian Maitner [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2118-9880>),
  Cory Merow [aut],
  Brad Boyle [aut],
  Xiao Feng [aut],
  Rethvick Sriram Yugendra Babu [aut],
  Brian Enquist [aut]
Maintainer: Brian Maitner <bmaitner@usf.edu>
Repository: CRAN
Date/Publication: 2026-08-25 07:10:02 UTC

# Generated by roxygen2: do not edit by hand

export(OpenRange_api_load_species)
export(OpenRange_api_species)
export(OpenRange_get_license)
export(OpenRange_get_stats)
export(OpenRange_list_scenarios)
export(OpenRange_load_species)
export(OpenRange_sf)
export(OpenRange_species)
import(RPostgreSQL)
import(sf)
importFrom(httr,GET)
importFrom(jsonlite,fromJSON)
importFrom(sf,st_as_sf)
importFrom(sf,st_write)

# Open Range

The goal of this repository is to provide access to a proof-of-concept Open Range database. The intent behind this Open Range database is to provide Open Access species range maps, along with associated metadata needed to judge map quality and suitability for different purposes. The maps within the database are licensed under a <a href="https://creativecommons.org/licenses/by/4.0/">CC-BY 4.0</a> license. The current database is focused on plants, primarily from the Americas, but we intend on expanding the scope in the future.

## Disclaimer

These maps are a <b>pre-alpha</b> version intended primarily as a <b>proof-of-concept</b> and have not been fully assessed or undergone a thorough QA/QC. While the associated metadata can be used to judge performance, these maps <b> should not be used</b> without critical evaluation by the user.

## Installation
This package can be installed using the `install_github` function in the `devtools` R package.

```
library(devtools)
install_github(repo = "MaitnerLab/OpenRange")


```

## ----load species-------------------------------------------------------------

library(OpenRange)


pcontorta <- OpenRange_load_species(species = "Pinus contorta")

plot(pcontorta[2])


## ----get metadata-------------------------------------------------------------

library(tidyverse)

# Grab the stats
  model_stats <- OpenRange_get_stats()

# Reformat them into a single table

  model_stats_wide <- bind_rows(model_stats$rangebagging %>%
                     mutate(algorithm = "rangebagging"),
                   model_stats$ppm %>%
                     mutate(algorithm = "ppm"),
                   model_stats$points %>%
                     mutate(algorithm = "points"))


# Get relevant model stats
  
  model_stats_wide %>%
    filter(model_id == pcontorta$model_id)->test

  test %>%
    select(cv_mean_train_npresence,cv_mean_test_n_presence,cv_mean_test_auc,cv_mean_train_auc)

#'Download license information
#'
#'OpenRange_get_license provides information on model licenses.
#' @param ... Additional arguments passed to internal functions.
#' @return Data.frame containing information on the license associated with the maps.
#' @examples \donttest{
#'
#' range_license_info <- OpenRange_get_license()
#'
#'}
#' @family range functions
#' @family metadata functions
#' @export
OpenRange_get_license <- function(...){
  
    return(ranges_sql(query = "select * from ranges.license ;", ...))
    
  
}

test_that("OpenRange_get_license returns df", {
  
  skip_if_offline(host = "r-project.org")
  
  scenarios <- OpenRange_get_license()
  
  # test below assume a data dictionary and will be skipped if one isn't returned
  expect_s3_class(object = scenarios,
                  class = "data.frame") 
  
})

#'Load Open Range maps for specified species.
#'
#'OpenRange_load_species returns spatial data for the specified species.
#' @param species A single species or a vector of species.
#' @param projection Numeric. What projection should maps be returned in?  4326 (default) or 3857 
#' @param scenario Which climate scenario(s) should be represented by maps?  See BIEN_ranges_list_scenarios for options. Set to NULL to download all.
#' @param default_only Logical. Should only default ranges be included? Default is TRUE.
#' @param ... Additional arguments passed to internal functions.
#' @return A SpatialPolygonsDataFrame containing range maps for the specified species.
#' @examples \donttest{
#' #' 
#' library(maps)
#' library(ggplot2)
#' library(sf)
#' 
#' species_vector <- c("Abies_lasiocarpa","Abies_amabilis")
#' abies_maps <- OpenRange_load_species(species = species_vector)
#' 
#' # To get all maps for a species, use "default = FALSE".
#' # Here, this returns maps with different thresholds from the same model
#' std_all <- OpenRange_load_species(species = "Stellaria debilis",
#'                                   default_only = FALSE)
#' 
#' #To just get the default map, use "default = TRUE"
#' std_default <- OpenRange_load_species(species = "Stellaria debilis",
#'                                       default_only = TRUE)
#' 
#' #get world map
#' 
#' world <- map("world", plot = FALSE, fill = TRUE)|>
#'   st_as_sf()
#' 
#' #Plotting ranges
#' 
#' ggplot(data = world)+
#'   geom_sf()+
#'   geom_sf(data = abies_maps,
#'           mapping = aes(fill = species),
#'           alpha=0.5)+
#'   coord_sf(xlim = st_bbox(abies_maps)[c(1,3)],
#'            ylim = st_bbox(abies_maps)[c(2,4)]) +
#'   theme_bw()
#' 
#'}
#' @family range functions
#' @export
OpenRange_load_species <- function(species,
                                   default_only = TRUE,
                                   projection = 4326,
                                   scenario = "present",
                                   ...){
  
  .is_char(species)
  .is_char(scenario)
  
  
  if(length(scenario)==1){
    if(scenario != "present"){default_only <- FALSE}
  }else{default_only <- FALSE}
  
  
  
  
  #make sure there are no spaces in the species names
  species <- gsub(" ","_",species)
  
  if(projection==4326){st_select<-"ST_AsText(geom2)"}
  if(projection==3857){st_select<-"ST_AsText(geom)"}
  
  if(default_only){
    default_map_select <- " and is_default = 1 "    
  }else{
    default_map_select <- ""
  }
  
  #If scenario is NULL, download ignore it and grab all
  
    if(!is.null(scenario)){
      
      scenario_select <- paste("AND scenario in (", paste(shQuote(scenario, type = "sh"),collapse = ', '), ")")
      
    }else{
      
      scenario_select <- ""
    }
  
  
  
  
  # set the query
  query <- paste("SELECT range_id,range_name,species_id,species,rangetype_id,source_id,source_name,run_id,run,batch_id,model_id,
                       model, statistics_unique_id, base_model, mod_type, model_moment, sampling, scenario_id, scenario, 
                       scenario_filecode, time_period, climate_model, rcp, threshold_id, threshold, background, is_default, 
                       basename, rel_path, ",st_select," 
                    FROM ranges.range 
                    WHERE species in (", paste(shQuote(species, type = "sh"),collapse = ', '), ")",
                  scenario_select, default_map_select, 
                 "ORDER BY species ;")
  
  
  df <- ranges_sql(query)
  
  if(length(df)==0){
    message("No species matched")
  }else{
    
    
    poly <- st_as_sf(x = df,
                     wkt = "st_astext",
                     crs = "epsg:4326")
    
    
  }#else
  
  return(poly) 
  
}

test_that("OpenRange_load_species returns sf", {
  
  skip_if_offline(host = "r-project.org")
  
  xs <- OpenRange_load_species(species = "Xanthium strumarium",
                                      default_only = TRUE)
  
  expect_s3_class(object = xs,class = "sf") 
  
  skip_if_not("sf" %in% class(xs))
  
  expect_equal(object = nrow(xs),expected = 1)  
  
})

#'Download range maps for given species using the API.
#'
#'OpenRange_api_species downloads range maps for the specified species.
#' @param species A single species.
#' @param include_id Logical. Should the range_id be appended to the file name?  Needed to save multiple maps per species.
#' @param directory Directory that range maps should be saved in.
#' @param ... Additional arguments passed to internal functions.
#' @note Details on the construction of BIEN range maps is available at http://bien.nceas.ucsb.edu/bien/biendata/bien-3/
#' @return NULL. Called for its side effect of downloading range maps.
#' @examples \donttest{
#'
#' library(OpenRange)
#' library(maps) #a convenient source of maps
#' library(sf)
#' library(ggplot2)
#' 
#' 
#' # Create temp directory
#' 
#' temp_dir <- file.path(tempdir(), "BIEN_temp")
#' 
#' #Download ranges
#' 
#' OpenRange_api_species(species = "Abies_amabilis",
#'                       directory = temp_dir,
#'                       include_id = TRUE) # saves ranges to a temporary directory
#' 
#' #Reading files
#' 
#' Abies_poly <- st_read(dsn = temp_dir,
#'                       layer = "Abies_amabilis_117684")
#' 
#' # Get a map to plot on
#' 
#' world <- map("world", plot = FALSE, fill = TRUE)|>
#'   st_as_sf()
#' 
#' #Plotting files
#' 
#' ggplot(data = world)+
#'   geom_sf()+
#'   geom_sf(data = Abies_poly,
#'           fill="green")+
#'   coord_sf(xlim = st_bbox(Abies_poly)[c(1,3)],
#'            ylim = st_bbox(Abies_poly)[c(2,4)]) +
#'   theme_bw()
#' 
#' #Getting data from the files
#' 
#' Abies_poly |>
#'   st_drop_geometry()
#' 
#' }
#' @family range functions
#' @importFrom sf st_as_sf st_write
#' @importFrom httr GET
#' @importFrom jsonlite fromJSON
#' @export
OpenRange_api_species <- function(species,
                              directory,
                              include_id = TRUE,
                              ...){
  
  #projection set (just WGS84 for now)
  
    projection <- 4326
  
  #check input formats
    .is_char(species)

  #make sure there are no spaces in the species names
    species <- gsub(" ","_",species)
    
  #make the URL
    url <- paste0("https://biendata.org/api/range/range-data?species=",species)
  
  # Download the data from the API
    req <- GET(url = url)
  
  # convert to raw - this throws an error
    raw_content <- rawToChar(req$content)
  
  # convert raw content from JSON to R object
    df <- fromJSON(raw_content)
  
  # check for messages
    
    if("message" %in% names(df)){
      
      message(df$message)
      return(invisible(NULL))
      
    }
  
      for(l in 1:length(df$species)){
        
        Species <- df$species[l]
        
        if(projection==4326){
          
          sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                               wkt = "st_astext",
                               crs = "epsg:4326")
          
          
        }
        
        if(projection==3857){
          
          sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                               wkt = "st_astext",
                               crs = "epsg:3857")
          
          
        }
        
        
        
        #save output
        if(include_id){
          
          suppressWarnings(st_write(obj = sp_range,
                                    dsn = file.path(directory),
                                    layer = paste(df$species[l],"_",df$range_id[l],sep=""),
                                    driver = "ESRI Shapefile",
                                    append = FALSE,
                                    quiet=TRUE))
          
        }else{
          
          suppressWarnings(st_write(obj = sp_range,
                                    dsn = file.path(directory),
                                    layer = paste(df$species[l]),
                                    driver = "ESRI Shapefile",
                                    append = FALSE,
                                    quiet=TRUE))
          
        }
        
        
      }#for species in df loop
  
    return(invisible(NULL))
  
  
    }#end fx

#'Download range maps that intersect a user-supplied sf object.
#'
#'OpenRange_sf extracts range maps that intersect a specified SpatialPolygons or SpatialPolygonsDataFrame object.
#' @param sf An object of class sf
#' @param crop.ranges Should the ranges be cropped to the focal area? Default is FALSE.
#' @param projection Numeric. What projection should maps be returned in?  4326 (default) or 3857 
#' @param scenario Which climate scenario should be represented by maps?  See BIEN_ranges_list_scenarios.
#' @param default_only Logical. Should only default ranges be included? Default is TRUE.
#' @template ranges_spatial
#' @return All range maps that intersect the user-supplied shapefile.
#' @examples \donttest{
#'
#'saguaro_poly <- OpenRange_load_species("Carnegiea gigantea")
#'
#'#Create a temp directory
#'temp_dir <- file.path(tempdir(), "BIEN_temp")
#'
#'#Get range maps for all species with ranges that overlap the range range of saguaros
#'OpenRange_sf(sf = saguaro_poly,
#'            directory = temp_dir)
#'
#'#Note that this will save many sfs to the directory (or working directory)
#' }
#' @family range functions
#' @import sf
#' @export
OpenRange_sf <- function(sf,
                         directory,
                         species.names.only = FALSE,
                         return.species.list = TRUE,
                         crop.ranges = FALSE,
                         include.gid = FALSE,
                         projection = 4326,
                         scenario = 'present',
                         default_only = TRUE,
                         ...){
  
  .is_log(return.species.list)
  .is_log(species.names.only)
  .is_log(crop.ranges)
  .is_log(include.gid)
  
  wkt <- sf |>
    st_geometry() |>
    st_as_text()
  
  
  if(species.names.only == FALSE){
    
    if(scenario != "present"){
      default_only = F
      }

    if(default_only){
      
      default_map_select <- " and is_default = 1 "
      
    }else{
      
      default_map_select <- ""
      
    }
    
    
    # set the query
    if(crop.ranges){
    if(projection==4326){
        query <- paste("SELECT range_id,range_name,species_id,species,rangetype_id,source_id,source_name,run_id,run,batch_id,model_id,
                           model, statistics_unique_id, base_model, mod_type, model_moment, sampling, scenario_id, scenario, 
                           scenario_filecode, time_period, climate_model, rcp, threshold_id, threshold, background, is_default, 
                           basename, rel_path, ST_AsText(ST_intersection(geom2,ST_GeographyFromText('SRID=4326;",paste(wkt),"'))),      
                        FROM ranges.range 
                        WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),geom2)
                        AND scenario in (", paste(shQuote(scenario, type = "sh"),collapse = ', '), ")", default_map_select, 
                       "ORDER BY species ;")
        }
    
    
    if(projection==3857){
      query <- paste("SELECT range_id,range_name,species_id,species,rangetype_id,source_id,source_name,run_id,run,batch_id,model_id,
                           model, statistics_unique_id, base_model, mod_type, model_moment, sampling, scenario_id, scenario, 
                           scenario_filecode, time_period, climate_model, rcp, threshold_id, threshold, background, is_default, 
                           basename, rel_path, ST_AsText(ST_intersection(geom,ST_GeographyFromText('SRID=3857;",paste(wkt),"'))),      
                        FROM ranges.range 
                        WHERE st_intersects(ST_GeographyFromText('SRID=3857;",paste(wkt),"'),geom)
                        AND scenario in (", paste(shQuote(scenario, type = "sh"),collapse = ', '), ")", default_map_select, 
                     "ORDER BY species ;")}  
    
    
    }else{
      
      if(projection==4326){
        
        query <- paste("SELECT range_id,range_name,species_id,species,rangetype_id,source_id,source_name,run_id,run,batch_id,model_id,
                           model, statistics_unique_id, base_model, mod_type, model_moment, sampling, scenario_id, scenario, 
                           scenario_filecode, time_period, climate_model, rcp, threshold_id, threshold, background, is_default, 
                           basename, rel_path, ST_AsText(geom2)       
                        FROM ranges.range 
                        WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),geom2)
                        AND scenario in (", paste(shQuote(scenario, type = "sh"),collapse = ', '), ")", default_map_select, 
                       "ORDER BY species ;")
      }
      
      
      if(projection==3857){
        query <- paste("SELECT range_id,range_name,species_id,species,rangetype_id,source_id,source_name,run_id,run,batch_id,model_id,
                           model, statistics_unique_id, base_model, mod_type, model_moment, sampling, scenario_id, scenario, 
                           scenario_filecode, time_period, climate_model, rcp, threshold_id, threshold, background, is_default, 
                           basename, rel_path, ST_AsText(geom)       
                        FROM ranges.range 
                        WHERE st_intersects(ST_GeographyFromText('SRID=3857;",paste(wkt),"'),geom)
                        AND scenario in (", paste(shQuote(scenario, type = "sh"),collapse = ', '), ")", default_map_select, 
                       "ORDER BY species ;")}   
      
      
    }
    
    # execute the query
    df <- ranges_sql(query = query, limit =10)
    
    if(length(df)==0){
      
      message("No species matched")
      
    }else{
      
      for(l in 1:length(df$species)){
        
        Species <- df$species[l]
        
        if(projection==3857){
          
          sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                               wkt = "st_astext",
                               crs = "epsg:3857")
          
        }
        
        if(projection==4326){
          
          sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                               wkt = "st_astext",
                               crs = "epsg:4326")
          
          
        }
        

          if(include.gid==T){

            st_write(obj = sp_range,
                     dsn = file.path(directory),
                     layer = paste(df$species[l],"_",df$range_id[l],sep=""),
                     driver = "ESRI Shapefile",
                     append = FALSE,
                     quiet=TRUE)
            
            
          }else{

            st_write(obj = sp_range,
                     dsn = file.path(directory),
                     layer = paste(df$species[l],sep=""),
                     driver = "ESRI Shapefile",
                     append = FALSE,
                     quiet=TRUE)
            
          }
          
          #save output
        }#if sp_range is not null  
      
      if(return.species.list){
        
        return(df[,2])
        
      }#if return.species.list  
      
    }#else
    
  }#species names only if statement
  
  if(species.names.only == TRUE){
    
    query <- paste("SELECT species
                   FROM ranges.range
                   WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),geom2)")
    
    # create query to retrieve
    
    df <- ranges_sql(query)
    
    if(length(df)==0){
      message("No species found")
    }else{
      return(df)
      
    }
    
  } #species.names.only ==TRUE


}

################################

#'Run an SQL query on the OpenRange database.
#'
#'ranges_sql is an internal function used to submit SQL queries.
#' @param query A PostgreSQL query.
#' @param user The username used to access the BIEN database
#' @param password The password associated with the username
#' @param ranges Alternative value to be substituted for "ranges" in queries when not NULL.
#' @param limit A limit on the number of records to be returned.  Should be a single number or NULL (the default).
#' @param return.query Should  the query used be returned rather than executed?  Default is FALSE
#' @param schema An alternative schema to be accessed.  Used for testing purposes.
#' @param db_name An alternate database to be used rather than the default, vegbien.
#' @param print.query Should  the query used be printed?  Default is FALSE
#' @return A dataframe returned by the query.
#' @keywords internal
#' @import RPostgreSQL
ranges_sql <- function(query,
                    user = 'public_bien',
                    password = 'bien_public',
                    ranges = NULL,
                    limit = NULL,
                    return.query = FALSE,
                    schema = "analytical_db",
                    db_name = 'vegbien',
                    print.query = FALSE){

  # user <- 'public_bien'
  # password <- 'bien_public'
  
  # .is_char(query)
  # .is_char(user)
  # .is_char(password)
  
  if(print.query){
    query <- gsub(pattern = "\n",replacement = "",query)
    query <- gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", query, perl=TRUE)
    print(query)
  }
  
  if(!is.null(ranges)){
    query<-gsub(pattern = "ranges",replacement = ranges,x = query)}
  
  if(!is.null(limit)){
    query<-gsub(pattern = " ;",replacement = paste(" LIMIT ",limit,";"),x = query)}
  
  if(return.query){
    query<-gsub(pattern = "\n",replacement = "",query)
    query<-gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", query, perl=TRUE)
    return(query)
  }
  
  
  host='vegbiendev.nceas.ucsb.edu'
  dbname= db_name
  user=user
  password=password
  # Name the database type that will be used
  drv <- DBI::dbDriver('PostgreSQL')
  # establish connection with database
  con <- DBI::dbConnect(drv, host=host, dbname=dbname, user=user, password = password)
  
  
  # create query to retrieve
  df <- DBI::dbGetQuery(con, statement = query);
  
  DBI::dbDisconnect(con)
  
  if(print.query){
    query<-gsub(pattern = "\n",replacement = "",query)
    query<-gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", query, perl=TRUE)
    print(query)
  }
  
  return(df)
  
}

# api urls

# URL for NSR API
#url <- "https://biendata.org/api/range/range-data?species=" # production
# N/A #testing

# Bad URLs for testing

# url <- "www.google.com"
# url <- "www.hisstank.com"

library("vcr") # *Required* as vcr is set up on loading

invisible(vcr::vcr_configure(
  dir = vcr::vcr_test_path("fixtures")
))

#Internal functions

#########################################
#Value checkers
#########################################
##################################
#'Check that value is logical
#'
#'Helper function to check data format.
#' @keywords internal
.is_log <- function(x) {
  if (!inherits(x, 'logical')) {
    stop(sys.call()[-1], " should be logical", call. = FALSE)
  }
}


##################################
#'Check that value is logical or null
#'
#'Helper function to check data format.
#' @keywords internal
.is_log_or_null <- function(x) {
  if (!inherits(x, c('logical','NULL'))) {
    stop(sys.call()[-1], " should be logical or NULL", call. = FALSE)
  }
}

###################################
#'Check that value is character
#'
#'Helper function to check data format.
#' @keywords internal
.is_char <- function(x) {
  if (!inherits(x ,c("character","NULL"))) {
    stop(sys.call()[-1]," should be character", call. = FALSE)
  }
}

###################################

#'Check that value is numeric
#'
#'Helper function to check data format.
#' @keywords internal
.is_num <- function(x) {
  if (!inherits(x ,'numeric')) {
    stop(sys.call()[-1]," should be numeric", call. = FALSE)
  }
}

#################################
#################################

#'Download information on available climate change scenarios
#'
#'OpenRange_list_scenarios provides information on climate scenarios with range projections available.
#' @param ... Additional arguments passed to internal functions.
#' @note Details on the construction of BIEN range maps is available at http://bien.nceas.ucsb.edu/bien/biendata/bien-3/
#' @return Data.frame containing climate change scenarios available in BIEN.
#' @examples \dontrun{
#'
#' cc_scenarios <- OpenRange_list_scenarios()
#'
#'}
#' @family range functions
#' @family metadata functions
#' @export
OpenRange_list_scenarios <- function(...){
  
  
  # set the query
  query <- paste("SELECT DISTINCT  scenario_id, scenario, scenario_filecode, time_period, climate_model, rcp  
                    FROM ranges.range ;")
  
  
  return(ranges_sql(query = query))
  
  
}

#'Download model performance statistics
#'
#'OpenRange_get_stats provides information on model performance.
#' @param ... Additional arguments passed to internal functions.
#' @return List object containing one data.frame for each of the three modeling frameworks.
#' @examples \donttest{
#'
#' range_stats <- OpenRange_get_stats()
#'
#'}
#' @family range functions
#' @family metadata functions
#' @export
OpenRange_get_stats <- function(...){
  
  out <- 
  list(rangebagging = ranges_sql(query = "select * from ranges.statistics_rangebag ;",...),
       ppm = ranges_sql(query = "select * from ranges.statistics_ppm ;",...),
       points = ranges_sql(query = "select * from ranges.statistics_points ;",...))
  

  
  
}

test_that("OpenRange_get_stats returns list", {
  
  skip_if_offline(host = "r-project.org")
  
  stats <- OpenRange_get_stats()
  
  # check for a list
  expect_identical(object = class(stats),expected = "list")
  
  #test below will be skipped if the stats object isn't a list
  
  skip_if_not(class(stats) == "list")
  
  expect_equal(object = length(stats),expected = 3)
  
})

#'Download range maps for given species.
#'
#'OpenRange_species extracts range maps for the specified species.
#' @param species A single species or a vector of species.
#' @param include_id Logical. Should the range_id be appended to the file name?  Needed to save multiple maps per species.
#' @param projection Numeric. What projection should maps be returned in?  4326 (default) or 3857 
#' @param scenario Which climate scenario should be represented by maps?  See BIEN_ranges_list_scenarios. Set to NULL to download all.
#' @param default_only Logical. Should only default ranges be included? Default is TRUE.
#' @param directory Directory that range maps should be saved in.
#' @param matched Return a list of taxa that were downloaded. Default is TRUE.
#' @param match_names_only Check for range maps for the taxa specified without downloading range maps. Default is FALSE.
#' @param ... Additional arguments passed to internal functions.
#' @note Details on the construction of BIEN range maps is available at http://bien.nceas.ucsb.edu/bien/biendata/bien-3/
#' @return Range maps for specified species.
#' @examples \donttest{
#' 
#'library(maps) #a convenient source of maps
#'library(sf)
#'
#'species_vector <- c("Abies_lasiocarpa","Abies_amabilis")
#'
#'#check whether the species are available
#'OpenRange_species(species_vector,match_names_only = TRUE)
#'
#'#Create a temp directory
#'temp_dir <- file.path(tempdir(), "BIEN_temp")
#'
#'#Download ranges
#'OpenRange_species(species = species_vector,
#'                  directory = temp_dir)#saves ranges to a temporary directory
#'
# OpenRange_species(species = "Abies_lasiocarpa",
#                   directory = temp_dir)
# 
#'#Reading files
#'
#'Abies_poly <- st_read(dsn = temp_dir,
#'                      layer = "Abies_amabilis_117684")
#'
#'#Plotting files
#'plot(Abies_poly[,1])#plots the range, but doesn't mean much without any reference
#'map('world', fill = TRUE, col = "grey")#plots a world map (WGS84 projection), in grey
#'plot(Abies_poly,col="forest green",add=TRUE) #adds the range of Abies lasiocarpa to the map
#'
#'#Getting data from the files
#'Abies_poly |>
#'  st_drop_geometry()
#'
#' }
#' @family range functions
#' @importFrom sf st_as_sf st_write
#' @export
OpenRange_species <- function(species,
                              directory,
                              default_only = TRUE,
                              matched = TRUE,
                              match_names_only = FALSE,
                              include_id = TRUE,
                              projection = 4326,
                              scenario = 'present', ...){
  
  .is_char(species)
  .is_log(matched)
  .is_log(match_names_only)
  .is_char(scenario)
  
  if(length(scenario)==1){
    if(scenario != "present"){default_only <- FALSE}
  }else{default_only <- FALSE}
  
  #make sure there are no spaces in the species names
  species <- gsub(" ","_",species)
  
  if(match_names_only==FALSE){
    
    if(projection==4326){st_select<-"ST_AsText(geom2)"}
    if(projection==3857){st_select<-"ST_AsText(geom)"}
    
    if(default_only){
      default_map_select <- " and is_default = 1 "    
    }else{
      default_map_select <- ""
    }
    
    #If scenario is NULL, download ignore it and grab all
    
    if(!is.null(scenario)){
      
      scenario_select <- paste("AND scenario in (", paste(shQuote(scenario, type = "sh"),collapse = ', '), ")")
      
    }else{
      
      scenario_select <- ""
    }
    
    
    
    # set the query
    query <- paste("SELECT range_id,range_name,species_id,species,rangetype_id,source_id,source_name,run_id,run,batch_id,model_id,
                    model, statistics_unique_id, base_model, mod_type, model_moment, sampling, scenario_id, scenario,
                    scenario_filecode, time_period, climate_model, rcp, threshold_id, threshold, background, is_default,
                    basename, rel_path, ",st_select," 
                    FROM ranges.range 
                    WHERE species in (", paste(shQuote(species, type = "sh"),collapse = ', '), ")", 
                    scenario_select, default_map_select, 
                   "ORDER BY species ;")
    
    df <- ranges_sql(query)
    
    if(length(df)==0){
      message("No species matched")
    }else{
      
      for(l in 1:length(df$species)){
        
        Species <- df$species[l]
        
        if(projection==4326){
          
          sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                               wkt = "st_astext",
                               crs = "epsg:4326")
          
          
        }
        
        if(projection==3857){
          
          sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                               wkt = "st_astext",
                               crs = "epsg:3857")
          
          
        }
        
        
        
        #save output
        if(include_id){
          
          suppressWarnings(st_write(obj = sp_range,
                                    dsn = file.path(directory),
                                    layer = paste(df$species[l],"_",df$range_id[l],sep=""),
                                    driver = "ESRI Shapefile",
                                    append = FALSE,
                                    quiet=TRUE))
          
        }else{
          
          suppressWarnings(st_write(obj = sp_range,
                                    dsn = file.path(directory),
                                    layer = paste(df$species[l]),
                                    driver = "ESRI Shapefile",
                                    append = FALSE,
                                    quiet=TRUE))
          
        }
        
        
      }#for species in df loop
    }#else
    
    #list matched species
    
    if(matched==TRUE){
      found <- as.data.frame(cbind(species,matrix(nrow=length(species),ncol=1,data="No")))
      colnames(found) <- c("Species","Range_map_downloaded?")
      found$`Range_map_downloaded?` <- as.character(found$`Range_map_downloaded?`)
      found$`Range_map_downloaded?`[which(species%in%df$species)] <- "Yes"
      return(found)
    }#matched = true
  }#match names only if statement
  
  if(match_names_only == TRUE){
    
    rangeQuery <- paste("SELECT species FROM ranges.range WHERE species in (", paste(shQuote(species, type = "sh"),collapse = ', '), ") ORDER BY species ;")
    # create query to retrieve
    df <- ranges_sql(rangeQuery,...)
    
    
    if(length(df)==0){
      message("No species matched")
    }else{
      found<-as.data.frame(cbind(species,matrix(nrow=length(species),ncol=1,data="No")))
      colnames(found)<-c("Species","Range_map_available?")
      found$`Range_map_available?`<-as.character(found$`Range_map_available?`)
      found$`Range_map_available?`[which(species%in%df$species)]<-"Yes"
      return(found)
      
    }
    
  } #matched_names_only ==TRUE
}

#'Load range maps for given species using the API.
#'
#'OpenRange_api_load_species extracts range maps for the specified species.
#' @param species A single species.
#' @param ... Additional arguments passed to internal functions.
#' @note Details on the construction of BIEN range maps is available at http://bien.nceas.ucsb.edu/bien/biendata/bien-3/
#' @return Range maps for specified species.
#' @examples \donttest{
#' 
#'library(maps) #a convenient source of maps
#'library(sf)
#'
#'temp_dir <- file.path(tempdir(), "BIEN_temp")
#'
#'#Download ranges
#'Abies_poly <- OpenRange_api_load_species(species = "Abies_amabilis")
#'
#'#Plotting files
#'plot(Abies_poly[,1])#plots the range, but doesn't mean much without any reference
#'map('world', fill = TRUE, col = "grey")#plots a world map (WGS84 projection), in grey
#'plot(Abies_poly,col="forest green",add=TRUE) #adds the range to the map
#'
#'#Getting data from the files
#'Abies_poly |>
#'  st_drop_geometry()
#'
#' }
#' @family range functions
#' @importFrom sf st_as_sf st_write
#' @importFrom httr GET
#' @importFrom jsonlite fromJSON
#' @export

OpenRange_api_load_species <- function(species,
                                  ...){
  
  #projection set (just WGS84 for now)
  
  projection <- 4326
  
  #check input formats
  .is_char(species)
  
  #make sure there are no spaces in the species names
  species <- gsub(" ","_",species)
  

  #make the URL
  url <- paste0("https://biendata.org/api/range/range-data?species=",species)
  
  # Download the data from the API
  req <- GET(url = url)
  
  # convert to raw - this throws an error
  raw_content <- rawToChar(req$content)
  
  # convert raw content from JSON to R object
  df <- fromJSON(raw_content)
  
  # check for messages
  
  if("message" %in% names(df)){
    
    message(df$message)
    return(invisible(NULL))
    
  }
  
  
  if(length(df)==0){
    message("No species matched")
  }else{
    
    
    poly <- st_as_sf(x = df,
                     wkt = "st_astext",
                     crs = "epsg:4326")
    
    
  }#else
  
  return(poly) 
  
}#end fx

test_that("OpenRange_api_load_species returns sf", {
  
  vcr::use_cassette("OpenRange_api_load_species", {
    
    xs <- OpenRange_api_load_species(species = "Xanthium strumarium")
    
  })
  
  expect_s3_class(object = xs,class = "sf") 
  
  skip_if_not("sf" %in% class(xs))
  
  expect_equal(object = nrow(xs),expected = 1)  
  

})

---
title: "OpenRange R package"
author: "Brian Maitner"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{OpenRange R package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  
---

# Open Range
Open Range is a database containing Open Access species range maps and associated metadata. The database is currently in a pre-alpha, proof-of-concept stage.  While the methods underlying the range maps are documented in the metadata, and the maps follow reasonable best practices, they have not been peer-reviewed.  They are provided to show a working example of what could be done with an Open Access range database, and to provide an example of what the underlying database might look like.  The maps should not be used without critical evaluation.

## Licensing
The maps currently contained in the Open Range database are available with a CC-BY license, but we note that additional maps using alternative licenses may be applied in the future.


### Accessing the maps
The easiest way to access the species range maps is by querying a species name.
```{r load species}

library(OpenRange)


pcontorta <- OpenRange_load_species(species = "Pinus contorta")

plot(pcontorta[2])

```

### Checking map quality
This is a bit cumbersome, but keep in mind this is a pre-alpha version.  For now, you can use the function OpenRange_get_stats

```{r get metadata}

library(tidyverse)

# Grab the stats
  model_stats <- OpenRange_get_stats()

# Reformat them into a single table

  model_stats_wide <- bind_rows(model_stats$rangebagging %>%
                     mutate(algorithm = "rangebagging"),
                   model_stats$ppm %>%
                     mutate(algorithm = "ppm"),
                   model_stats$points %>%
                     mutate(algorithm = "points"))


# Get relevant model stats
  
  model_stats_wide %>%
    filter(model_id == pcontorta$model_id)->test

  test %>%
    select(cv_mean_train_npresence,cv_mean_test_n_presence,cv_mean_test_auc,cv_mean_train_auc)


```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ranges_sql.R
\name{ranges_sql}
\alias{ranges_sql}
\title{Run an SQL query on the OpenRange database.}
\usage{
ranges_sql(
  query,
  user = "public_bien",
  password = "bien_public",
  ranges = NULL,
  limit = NULL,
  return.query = FALSE,
  schema = "analytical_db",
  db_name = "vegbien",
  print.query = FALSE
)
}
\arguments{
\item{query}{A PostgreSQL query.}

\item{user}{The username used to access the BIEN database}

\item{password}{The password associated with the username}

\item{ranges}{Alternative value to be substituted for "ranges" in queries when not NULL.}

\item{limit}{A limit on the number of records to be returned.  Should be a single number or NULL (the default).}

\item{return.query}{Should  the query used be returned rather than executed?  Default is FALSE}

\item{schema}{An alternative schema to be accessed.  Used for testing purposes.}

\item{db_name}{An alternate database to be used rather than the default, vegbien.}

\item{print.query}{Should  the query used be printed?  Default is FALSE}
}
\value{
A dataframe returned by the query.
}
\description{
ranges_sql is an internal function used to submit SQL queries.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OpenRange_sf.R
\name{OpenRange_sf}
\alias{OpenRange_sf}
\title{Download range maps that intersect a user-supplied sf object.}
\usage{
OpenRange_sf(
  sf,
  directory,
  species.names.only = FALSE,
  return.species.list = TRUE,
  crop.ranges = FALSE,
  include.gid = FALSE,
  projection = 4326,
  scenario = "present",
  default_only = TRUE,
  ...
)
}
\arguments{
\item{sf}{An object of class sf}

\item{directory}{Directory that range maps should be saved in.}

\item{species.names.only}{Return species names rather than spatial data? Default is FALSE.}

\item{return.species.list}{Should a species list be returned in addition to downloading range maps?  Default is FALSE}

\item{crop.ranges}{Should the ranges be cropped to the focal area? Default is FALSE.}

\item{include.gid}{Should the files returned have a unique GID appended to them? This is needed if downloading multiple maps for the same species.}

\item{projection}{Numeric. What projection should maps be returned in?  4326 (default) or 3857}

\item{scenario}{Which climate scenario should be represented by maps?  See BIEN_ranges_list_scenarios.}

\item{default_only}{Logical. Should only default ranges be included? Default is TRUE.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
All range maps that intersect the user-supplied shapefile.
}
\description{
OpenRange_sf extracts range maps that intersect a specified SpatialPolygons or SpatialPolygonsDataFrame object.
}
\note{
Details on the construction of BIEN range maps is available at https://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\donttest{

saguaro_poly <- OpenRange_load_species("Carnegiea gigantea")

#Create a temp directory
temp_dir <- file.path(tempdir(), "BIEN_temp")

#Get range maps for all species with ranges that overlap the range range of saguaros
OpenRange_sf(sf = saguaro_poly,
           directory = temp_dir)

#Note that this will save many sfs to the directory (or working directory)
}
}
\seealso{
Other range functions: 
\code{\link{OpenRange_api_load_species}()},
\code{\link{OpenRange_api_species}()},
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_get_stats}()},
\code{\link{OpenRange_list_scenarios}()},
\code{\link{OpenRange_load_species}()},
\code{\link{OpenRange_species}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OpenRange_get_stats.R
\name{OpenRange_get_stats}
\alias{OpenRange_get_stats}
\title{Download model performance statistics}
\usage{
OpenRange_get_stats(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
List object containing one data.frame for each of the three modeling frameworks.
}
\description{
OpenRange_get_stats provides information on model performance.
}
\examples{
\donttest{

range_stats <- OpenRange_get_stats()

}
}
\seealso{
Other range functions: 
\code{\link{OpenRange_api_load_species}()},
\code{\link{OpenRange_api_species}()},
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_list_scenarios}()},
\code{\link{OpenRange_load_species}()},
\code{\link{OpenRange_sf}()},
\code{\link{OpenRange_species}()}

Other metadata functions: 
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_list_scenarios}()}
}
\concept{metadata functions}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OpenRange_load_species.R
\name{OpenRange_load_species}
\alias{OpenRange_load_species}
\title{Load Open Range maps for specified species.}
\usage{
OpenRange_load_species(
  species,
  default_only = TRUE,
  projection = 4326,
  scenario = "present",
  ...
)
}
\arguments{
\item{species}{A single species or a vector of species.}

\item{default_only}{Logical. Should only default ranges be included? Default is TRUE.}

\item{projection}{Numeric. What projection should maps be returned in?  4326 (default) or 3857}

\item{scenario}{Which climate scenario(s) should be represented by maps?  See BIEN_ranges_list_scenarios for options. Set to NULL to download all.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A SpatialPolygonsDataFrame containing range maps for the specified species.
}
\description{
OpenRange_load_species returns spatial data for the specified species.
}
\examples{
\donttest{
#' 
library(maps)
library(ggplot2)
library(sf)

species_vector <- c("Abies_lasiocarpa","Abies_amabilis")
abies_maps <- OpenRange_load_species(species = species_vector)

# To get all maps for a species, use "default = FALSE".
# Here, this returns maps with different thresholds from the same model
std_all <- OpenRange_load_species(species = "Stellaria debilis",
                                  default_only = FALSE)

#To just get the default map, use "default = TRUE"
std_default <- OpenRange_load_species(species = "Stellaria debilis",
                                      default_only = TRUE)

#get world map

world <- map("world", plot = FALSE, fill = TRUE)|>
  st_as_sf()

#Plotting ranges

ggplot(data = world)+
  geom_sf()+
  geom_sf(data = abies_maps,
          mapping = aes(fill = species),
          alpha=0.5)+
  coord_sf(xlim = st_bbox(abies_maps)[c(1,3)],
           ylim = st_bbox(abies_maps)[c(2,4)]) +
  theme_bw()

}
}
\seealso{
Other range functions: 
\code{\link{OpenRange_api_load_species}()},
\code{\link{OpenRange_api_species}()},
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_get_stats}()},
\code{\link{OpenRange_list_scenarios}()},
\code{\link{OpenRange_sf}()},
\code{\link{OpenRange_species}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OpenRange_species.R
\name{OpenRange_species}
\alias{OpenRange_species}
\title{Download range maps for given species.}
\usage{
OpenRange_species(
  species,
  directory,
  default_only = TRUE,
  matched = TRUE,
  match_names_only = FALSE,
  include_id = TRUE,
  projection = 4326,
  scenario = "present",
  ...
)
}
\arguments{
\item{species}{A single species or a vector of species.}

\item{directory}{Directory that range maps should be saved in.}

\item{default_only}{Logical. Should only default ranges be included? Default is TRUE.}

\item{matched}{Return a list of taxa that were downloaded. Default is TRUE.}

\item{match_names_only}{Check for range maps for the taxa specified without downloading range maps. Default is FALSE.}

\item{include_id}{Logical. Should the range_id be appended to the file name?  Needed to save multiple maps per species.}

\item{projection}{Numeric. What projection should maps be returned in?  4326 (default) or 3857}

\item{scenario}{Which climate scenario should be represented by maps?  See BIEN_ranges_list_scenarios. Set to NULL to download all.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Range maps for specified species.
}
\description{
OpenRange_species extracts range maps for the specified species.
}
\note{
Details on the construction of BIEN range maps is available at http://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\donttest{

library(maps) #a convenient source of maps
library(sf)

species_vector <- c("Abies_lasiocarpa","Abies_amabilis")

#check whether the species are available
OpenRange_species(species_vector,match_names_only = TRUE)

#Create a temp directory
temp_dir <- file.path(tempdir(), "BIEN_temp")

#Download ranges
OpenRange_species(species = species_vector,
                 directory = temp_dir)#saves ranges to a temporary directory

#Reading files

Abies_poly <- st_read(dsn = temp_dir,
                     layer = "Abies_amabilis_117684")

#Plotting files
plot(Abies_poly[,1])#plots the range, but doesn't mean much without any reference
map('world', fill = TRUE, col = "grey")#plots a world map (WGS84 projection), in grey
plot(Abies_poly,col="forest green",add=TRUE) #adds the range of Abies lasiocarpa to the map

#Getting data from the files
Abies_poly |>
 st_drop_geometry()

}
}
\seealso{
Other range functions: 
\code{\link{OpenRange_api_load_species}()},
\code{\link{OpenRange_api_species}()},
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_get_stats}()},
\code{\link{OpenRange_list_scenarios}()},
\code{\link{OpenRange_load_species}()},
\code{\link{OpenRange_sf}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OpenRange_api_species.R
\name{OpenRange_api_species}
\alias{OpenRange_api_species}
\title{Download range maps for given species using the API.}
\usage{
OpenRange_api_species(species, directory, include_id = TRUE, ...)
}
\arguments{
\item{species}{A single species.}

\item{directory}{Directory that range maps should be saved in.}

\item{include_id}{Logical. Should the range_id be appended to the file name?  Needed to save multiple maps per species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
NULL. Called for its side effect of downloading range maps.
}
\description{
OpenRange_api_species downloads range maps for the specified species.
}
\note{
Details on the construction of BIEN range maps is available at http://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\donttest{

library(OpenRange)
library(maps) #a convenient source of maps
library(sf)
library(ggplot2)


# Create temp directory

temp_dir <- file.path(tempdir(), "BIEN_temp")

#Download ranges

OpenRange_api_species(species = "Abies_amabilis",
                      directory = temp_dir,
                      include_id = TRUE) # saves ranges to a temporary directory

#Reading files

Abies_poly <- st_read(dsn = temp_dir,
                      layer = "Abies_amabilis_117684")

# Get a map to plot on

world <- map("world", plot = FALSE, fill = TRUE)|>
  st_as_sf()

#Plotting files

ggplot(data = world)+
  geom_sf()+
  geom_sf(data = Abies_poly,
          fill="green")+
  coord_sf(xlim = st_bbox(Abies_poly)[c(1,3)],
           ylim = st_bbox(Abies_poly)[c(2,4)]) +
  theme_bw()

#Getting data from the files

Abies_poly |>
  st_drop_geometry()

}
}
\seealso{
Other range functions: 
\code{\link{OpenRange_api_load_species}()},
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_get_stats}()},
\code{\link{OpenRange_list_scenarios}()},
\code{\link{OpenRange_load_species}()},
\code{\link{OpenRange_sf}()},
\code{\link{OpenRange_species}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OpenRange_list_scenarios.R
\name{OpenRange_list_scenarios}
\alias{OpenRange_list_scenarios}
\title{Download information on available climate change scenarios}
\usage{
OpenRange_list_scenarios(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
Data.frame containing climate change scenarios available in BIEN.
}
\description{
OpenRange_list_scenarios provides information on climate scenarios with range projections available.
}
\note{
Details on the construction of BIEN range maps is available at http://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\dontrun{

cc_scenarios <- OpenRange_list_scenarios()

}
}
\seealso{
Other range functions: 
\code{\link{OpenRange_api_load_species}()},
\code{\link{OpenRange_api_species}()},
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_get_stats}()},
\code{\link{OpenRange_load_species}()},
\code{\link{OpenRange_sf}()},
\code{\link{OpenRange_species}()}

Other metadata functions: 
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_get_stats}()}
}
\concept{metadata functions}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OpenRange_api_load_species.R
\name{OpenRange_api_load_species}
\alias{OpenRange_api_load_species}
\title{Load range maps for given species using the API.}
\usage{
OpenRange_api_load_species(species, ...)
}
\arguments{
\item{species}{A single species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Range maps for specified species.
}
\description{
OpenRange_api_load_species extracts range maps for the specified species.
}
\note{
Details on the construction of BIEN range maps is available at http://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\donttest{

library(maps) #a convenient source of maps
library(sf)

temp_dir <- file.path(tempdir(), "BIEN_temp")

#Download ranges
Abies_poly <- OpenRange_api_load_species(species = "Abies_amabilis")

#Plotting files
plot(Abies_poly[,1])#plots the range, but doesn't mean much without any reference
map('world', fill = TRUE, col = "grey")#plots a world map (WGS84 projection), in grey
plot(Abies_poly,col="forest green",add=TRUE) #adds the range to the map

#Getting data from the files
Abies_poly |>
 st_drop_geometry()

}
}
\seealso{
Other range functions: 
\code{\link{OpenRange_api_species}()},
\code{\link{OpenRange_get_license}()},
\code{\link{OpenRange_get_stats}()},
\code{\link{OpenRange_list_scenarios}()},
\code{\link{OpenRange_load_species}()},
\code{\link{OpenRange_sf}()},
\code{\link{OpenRange_species}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internals.R
\name{.is_char}
\alias{.is_char}
\title{Check that value is character}
\usage{
.is_char(x)
}
\description{
Helper function to check data format.
}
\keyword{internal}
